#
# This is a Shiny web application. You can run the application by clicking
# the 'Run App' button above.
#
# Find out more about building applications with Shiny here:
#
#    http://shiny.rstudio.com/
#


# All the packages I need
library(shiny)
library(tidyverse)
library(janitor)
library(lomb)
library(rsconnect)


# Define UI for application that draws a histogram
ui <- fluidPage(

    # Application title
    titlePanel("Young Stellar Objects in the Large Magellanic Cloud"),

    # Sidebar with a slider input for number of bins 
    sidebarLayout(
        position = "right",
        sidebarPanel(
        h2("Object Selection"),
        br(),
        # Creates a dropdown menu to select an object
        selectInput("seq_var", 
                    label = "Choose an Object to View",
                    choices = list("226" = 226, "263" = 263, "270" = 270, "290" = 290, 
                                   "301" = 301, "314" = 314, "315" = 315, "327" = 327, 
                                   "735" = 735, "741" = 741, "753" = 753, "799" = 799,
                                   "874" = 874, "1108" = 1108),
                    selected = "315")
        ),

        # Show a plot of the generated distribution
        mainPanel(
           # Creating the facet plot output and plot description
           plotOutput("facet_plot"),
           br(),
           p("This plot shows the brightness (in magnitudes) of an object in two different wavelengths (w1mpro and w2mpro) in groups of observations taken close in time, which are roughly six months apart and are represented by the Class Code. Objects that are non-variable appear globular in shape with observations scattered evenly compared to the overall distribution. Variable objects result in a linear shape as the brightness increases or decreases in both wavelengths simultaneously."),
           br(),
           # Creating the lomb-scargle plot and plot description
           plotOutput("lomb_scargle_plot"),
           br(),
           p("The Lomb-Scargle algorithm determines the periodicity of observations at infrequent time intervals, which describes this data set. The plot above shows the power at each period (in days) in the range calculated. Peaks that are statistically significant are above the dotted line. A taller and wider peak indicates a higher power at that period."),
           br(),
           # Creating the folded light curve plot and plot description
           plotOutput("folded_light_curve_plot"),
           p("The folded light curve takes the period determined by the Lomb-Scargle algorithm and adjusts the points to illustrate the variation in stellar brightness over one period. Non-variable objects do not show any pattern and tend to have scattered points. Variable objects with a relatively constant periodicity have folded curves where points seem to follow a sinusoidal function. Objects with a somewhat variable periodicity appear to look shifted when compared to other observations. There are also instances where material is accreted and a burst of light is emitted as a product of a random event. When periods are close to 180 days (the time difference between one Class Code to the next), it is perhaps an artifact of the sampling delay and random events rather than exhibiting periodic behavior. An example of this is when the Folded Light Curve produces two clusters separated by approximately 180 days."),
           br()
           
        )
)
)


# Define server logic required to draw my three plots
server <- function(input, output) {
  
    # Allowing myself to upload a file greater than 5MB to r shiny 
    options(shiny.maxRequestSize=30*1024^2)
  
    # Creating a list of hand-picked colors for my plots
    selected_colors <- list("#3D0202", "#D91212", "#E33D1F", "#5F4706", "#F2980E", 
                          "#E8C20A", "#0AE8E0", "#0AA7CF", "#0A54CF", "#BB40DC", 
                          "#6440DC", "#500863")
    
    # Reading in the data
    lmc_neowise_var <- read_csv("data/LMC_NEOWISE_VAR.csv")
    
    # Creating the ar_class column and filtering out missing data
    lmc_neowise_var_class <- lmc_neowise_var %>%
        mutate(ar_class = as.factor(trunc((mjd - 56700) / 180.125))) %>%
        clean_names() %>%
        filter(!is.na(w1mpro)) %>%
        filter(!is.na(w2mpro))


    
    
    # Defining the Objects as a list of doubles
    list_objects <- list(226, 263, 270, 290, 301, 314, 315, 327, 330,
                         353, 393, 407, 408, 439, 461, 481, 492, 500, 
                         535, 541, 559, 611, 656, 735, 741, 753, 799, 
                         874, 1108)
    
    # Creating an empty list for the loop to store values in
    file_list <- list()
    
    
    # Writing a for loop to create a new column of adjusted mjds
    # Uses the lomb-scargle information to create adjustment
    for (i in list_objects){
      lmc_neowise_var_class_filter <- lmc_neowise_var_class %>% 
        filter(seq == i) %>% 
        select(c(mjd, w1mpro, seq, ar_class))
      
      
      
      lmc.spec <- lsp(lmc_neowise_var_class_filter$w1mpro, times = lmc_neowise_var_class_filter$mjd, from = 5, ofac = 5, type = "period")
      
      lmc.spec$peak.at[1]
      
      file_list[[i]] <-  lmc_neowise_var_class_filter %>% 
        mutate(adj_mjd = mjd - min(lmc_neowise_var_class_filter %>% 
                                     select(mjd)) - trunc((mjd - min(lmc_neowise_var_class_filter %>% 
                                                                       select(mjd)))/lmc.spec$peak.at[1])*lmc.spec$peak.at[1]
        )
      
      
    } 
    
    # Binding all of the created tibbles together into one collective tibble
    lmc_neowise_var_class_adj <- bind_rows(file_list)

    
   # Creating the first facet plot
   output$facet_plot <- renderPlot(
     # Filters out for only the object selected 
     lmc_neowise_var_class %>% 
       filter(seq == input$seq_var) %>% 
       ggplot(aes(x = w1mpro, y = w2mpro, color = ar_class)) +
       geom_point(data = lmc_neowise_facet_plot_background <- dplyr::select(lmc_neowise_var_class%>% 
                                                                              filter(seq == input$seq_var), -ar_class), color = "grey80", size = 1) +
       geom_point() +
       # Creates the facet plot and includes aesthetic customization for plot
       facet_wrap(~ar_class) +
       scale_fill_manual(
         values = selected_colors
       ) +
       theme_minimal() +
       labs(
         color = "Class Code",
         title = "Faceted Data"
       ) +
       theme(
         plot.title = element_text(size = 18, family = "serif", face = "bold", hjust = 0.5),
         axis.title = element_text(family = "serif"),
       )
   )

   # Creating the lomb-scargle plot for the selected object 
   output$lomb_scargle_plot <- renderPlot(
     lmc_neowise_var_class %>%
       filter(seq == input$seq_var) %>% select(mjd, w1mpro) %>%
       lsp(from = 2, ofac = 5, type = "period")
   )
   
   # Creates the folded light curve from the tibble created in this server section
   output$folded_light_curve_plot <- renderPlot(
     lmc_neowise_var_class_adj %>% 
       filter(seq == input$seq_var) %>% 
       ggplot(aes(x = adj_mjd, y = w1mpro, fill = ar_class)) +
       geom_point(shape = 21, color = "black", size = 3, alpha = 0.5) +
       scale_fill_manual(
         values = selected_colors
       ) +
       theme_minimal() +
       labs(
         fill = "Class Code",
         title = "Adjusted Light Curve",
         x = "Adjusted mjd (days)"
       ) +
       theme(
         plot.title = element_text(size = 14, family = "serif", face = "bold", hjust = 0.5),
         axis.title = element_text(family = "serif")
       )
     

   )

}

# Run the application 
shinyApp(ui = ui, server = server)

